@@ -1,12 +1,9 @@
 """
 Functions for identifying peaks in signals.
 """
-from __future__ import division, print_function, absolute_import
-
 import math
 import numpy as np
 
-from scipy._lib.six import xrange
 from scipy.signal.wavelets import cwt, ricker
 from scipy.stats import scoreatpercentile
 
@@ -37,14 +34,14 @@ def _boolrelextrema(data, comparator, axis=0, order=1, mode='clip'):
         Function to use to compare two data points.
         Should take two arrays as arguments.
     axis : int, optional
-        Axis over which to select from `data`.  Default is 0.
+        Axis over which to select from `data`. Default is 0.
     order : int, optional
         How many points on each side to use for the comparison
         to consider ``comparator(n,n+x)`` to be True.
     mode : str, optional
-        How the edges of the vector are treated.  'wrap' (wrap around) or
+        How the edges of the vector are treated. 'wrap' (wrap around) or
         'clip' (treat overflow as the same as the last (or first) element).
-        Default 'clip'.  See numpy.take
+        Default 'clip'. See numpy.take.
 
     Returns
     -------
@@ -71,7 +68,7 @@ def _boolrelextrema(data, comparator, axis=0, order=1, mode='clip'):
 
     results = np.ones(data.shape, dtype=bool)
     main = data.take(locs, axis=axis, mode=mode)
-    for shift in xrange(1, order + 1):
+    for shift in range(1, order + 1):
         plus = data.take(locs + shift, axis=axis, mode=mode)
         minus = data.take(locs - shift, axis=axis, mode=mode)
         results &= comparator(main, plus)
@@ -90,7 +87,7 @@ def argrelmin(data, axis=0, order=1, mode='clip'):
     data : ndarray
         Array in which to find the relative minima.
     axis : int, optional
-        Axis over which to select from `data`.  Default is 0.
+        Axis over which to select from `data`. Default is 0.
     order : int, optional
         How many points on each side to use for the comparison
         to consider ``comparator(n, n+x)`` to be True.
@@ -98,14 +95,14 @@ def argrelmin(data, axis=0, order=1, mode='clip'):
         How the edges of the vector are treated.
         Available options are 'wrap' (wrap around) or 'clip' (treat overflow
         as the same as the last (or first) element).
-        Default 'clip'. See numpy.take
+        Default 'clip'. See numpy.take.
 
     Returns
     -------
     extrema : tuple of ndarrays
-        Indices of the minima in arrays of integers.  ``extrema[k]`` is
-        the array of indices of axis `k` of `data`.  Note that the
-        return value is a tuple even when `data` is one-dimensional.
+        Indices of the minima in arrays of integers. ``extrema[k]`` is
+        the array of indices of axis `k` of `data`. Note that the
+        return value is a tuple even when `data` is 1-D.
 
     See Also
     --------
@@ -113,10 +110,10 @@ def argrelmin(data, axis=0, order=1, mode='clip'):
 
     Notes
     -----
-    This function uses `argrelextrema` with np.less as comparator. Therefore it
+    This function uses `argrelextrema` with np.less as comparator. Therefore, it
     requires a strict inequality on both sides of a value to consider it a
     minimum. This means flat minima (more than one sample wide) are not detected.
-    In case of one-dimensional `data` `find_peaks` can be used to detect all
+    In case of 1-D `data` `find_peaks` can be used to detect all
     local minima, including flat ones, by calling it with negated `data`.
 
     .. versionadded:: 0.11.0
@@ -147,7 +144,7 @@ def argrelmax(data, axis=0, order=1, mode='clip'):
     data : ndarray
         Array in which to find the relative maxima.
     axis : int, optional
-        Axis over which to select from `data`.  Default is 0.
+        Axis over which to select from `data`. Default is 0.
     order : int, optional
         How many points on each side to use for the comparison
         to consider ``comparator(n, n+x)`` to be True.
@@ -155,14 +152,14 @@ def argrelmax(data, axis=0, order=1, mode='clip'):
         How the edges of the vector are treated.
         Available options are 'wrap' (wrap around) or 'clip' (treat overflow
         as the same as the last (or first) element).
-        Default 'clip'.  See `numpy.take`.
+        Default 'clip'. See `numpy.take`.
 
     Returns
     -------
     extrema : tuple of ndarrays
-        Indices of the maxima in arrays of integers.  ``extrema[k]`` is
-        the array of indices of axis `k` of `data`.  Note that the
-        return value is a tuple even when `data` is one-dimensional.
+        Indices of the maxima in arrays of integers. ``extrema[k]`` is
+        the array of indices of axis `k` of `data`. Note that the
+        return value is a tuple even when `data` is 1-D.
 
     See Also
     --------
@@ -170,10 +167,10 @@ def argrelmax(data, axis=0, order=1, mode='clip'):
 
     Notes
     -----
-    This function uses `argrelextrema` with np.greater as comparator. Therefore
+    This function uses `argrelextrema` with np.greater as comparator. Therefore,
     it  requires a strict inequality on both sides of a value to consider it a
     maximum. This means flat maxima (more than one sample wide) are not detected.
-    In case of one-dimensional `data` `find_peaks` can be used to detect all
+    In case of 1-D `data` `find_peaks` can be used to detect all
     local maxima, including flat ones.
 
     .. versionadded:: 0.11.0
@@ -206,21 +203,21 @@ def argrelextrema(data, comparator, axis=0, order=1, mode='clip'):
         Function to use to compare two data points.
         Should take two arrays as arguments.
     axis : int, optional
-        Axis over which to select from `data`.  Default is 0.
+        Axis over which to select from `data`. Default is 0.
     order : int, optional
         How many points on each side to use for the comparison
         to consider ``comparator(n, n+x)`` to be True.
     mode : str, optional
-        How the edges of the vector are treated.  'wrap' (wrap around) or
+        How the edges of the vector are treated. 'wrap' (wrap around) or
         'clip' (treat overflow as the same as the last (or first) element).
-        Default is 'clip'.  See `numpy.take`.
+        Default is 'clip'. See `numpy.take`.
 
     Returns
     -------
     extrema : tuple of ndarrays
-        Indices of the maxima in arrays of integers.  ``extrema[k]`` is
-        the array of indices of axis `k` of `data`.  Note that the
-        return value is a tuple even when `data` is one-dimensional.
+        Indices of the maxima in arrays of integers. ``extrema[k]`` is
+        the array of indices of axis `k` of `data`. Note that the
+        return value is a tuple even when `data` is 1-D.
 
     See Also
     --------
@@ -251,7 +248,7 @@ def argrelextrema(data, comparator, axis=0, order=1, mode='clip'):
 
 
 def _arg_x_as_expected(value):
-    """Ensure argument `x` is a 1D C-contiguous array of dtype('float64').
+    """Ensure argument `x` is a 1-D C-contiguous array of dtype('float64').
 
     Used in `find_peaks`, `peak_prominences` and `peak_widths` to make `x`
     compatible with the signature of the wrapped Cython functions.
@@ -259,16 +256,16 @@ def _arg_x_as_expected(value):
     Returns
     -------
     value : ndarray
-        A one-dimensional C-contiguous array with dtype('float64').
+        A 1-D C-contiguous array with dtype('float64').
     """
     value = np.asarray(value, order='C', dtype=np.float64)
     if value.ndim != 1:
-        raise ValueError('`x` must be a 1D array')
+        raise ValueError('`x` must be a 1-D array')
     return value
 
 
 def _arg_peaks_as_expected(value):
-    """Ensure argument `peaks` is a 1D C-contiguous array of dtype('intp').
+    """Ensure argument `peaks` is a 1-D C-contiguous array of dtype('intp').
 
     Used in `peak_prominences` and `peak_widths` to make `peaks` compatible
     with the signature of the wrapped Cython functions.
@@ -276,7 +273,7 @@ def _arg_peaks_as_expected(value):
     Returns
     -------
     value : ndarray
-        A one-dimensional C-contiguous array with dtype('intp').
+        A 1-D C-contiguous array with dtype('intp').
     """
     value = np.asarray(value)
     if value.size == 0:
@@ -286,10 +283,10 @@ def _arg_peaks_as_expected(value):
         # Safely convert to C-contiguous array of type np.intp
         value = value.astype(np.intp, order='C', casting='safe',
                              subok=False, copy=False)
-    except TypeError:
-        raise TypeError("cannot safely cast `peaks` to dtype('intp')")
+    except TypeError as e:
+        raise TypeError("cannot safely cast `peaks` to dtype('intp')") from e
     if value.ndim != 1:
-        raise ValueError('`peaks` must be a 1D array')
+        raise ValueError('`peaks` must be a 1-D array')
     return value
 
 
@@ -355,7 +352,7 @@ def peak_prominences(x, peaks, wlen=None):
     Warns
     -----
     PeakPropertyWarning
-        For indices in `peaks` that don't point to valid local maxima in `x`
+        For indices in `peaks` that don't point to valid local maxima in `x`,
         the returned prominence will be 0 and this warning is raised. This
         also happens if `wlen` is smaller than the plateau size of a peak.
 
@@ -391,10 +388,10 @@ def peak_prominences(x, peaks, wlen=None):
     parameter `wlen` which restricts the algorithm to a window around the
     current peak and can shorten the calculation time if the window length is
     short in relation to `x`.
-    However this may stop the algorithm from finding the true global contour
-    line if the peak's true bases are outside this window. Instead a higher
+    However, this may stop the algorithm from finding the true global contour
+    line if the peak's true bases are outside this window. Instead, a higher
     contour line is found within the restricted window leading to a smaller
-    calculated prominence. In practice this is only relevant for the highest set
+    calculated prominence. In practice, this is only relevant for the highest set
     of peaks in `x`. This behavior may even be used intentionally to calculate
     "local" prominences.
 
@@ -443,17 +440,17 @@ def peak_prominences(x, peaks, wlen=None):
     (array([3.]), array([2]), array([6]))
 
     Note how the peak at index 3 of the same height is not considered as a
-    border while searching for the left base. Instead two minima at 0 and 2
+    border while searching for the left base. Instead, two minima at 0 and 2
     are found in which case the one closer to the evaluated peak is always
-    chosen. On the right side however the base must be placed at 6 because the
+    chosen. On the right side, however, the base must be placed at 6 because the
     higher peak represents the right border to the evaluated area.
 
     >>> peak_prominences(x, peaks, wlen=3.1)
     (array([2.]), array([4]), array([6]))
 
-    Here we restricted the algorithm to a window from 3 to 7 (the length is 5
-    samples because `wlen` was rounded up to the next odd integer). Thus the
-    only two candidates in the evaluated area are the two neighbouring samples
+    Here, we restricted the algorithm to a window from 3 to 7 (the length is 5
+    samples because `wlen` was rounded up to the next odd integer). Thus, the
+    only two candidates in the evaluated area are the two neighboring samples
     and a smaller prominence is calculated.
     """
     x = _arg_x_as_expected(x)
@@ -482,7 +479,7 @@ def peak_widths(x, peaks, rel_height=0.5, prominence_data=None, wlen=None):
         height. Must be at least 0. See notes for further explanation.
     prominence_data : tuple, optional
         A tuple of three arrays matching the output of `peak_prominences` when
-        called with the same arguments `x` and `peaks`. This data is calculated
+        called with the same arguments `x` and `peaks`. This data are calculated
         internally if not provided.
     wlen : int, optional
         A window length in samples passed to `peak_prominences` as an optional
@@ -545,7 +542,7 @@ def peak_widths(x, peaks, rel_height=0.5, prominence_data=None, wlen=None):
 
     As shown above to calculate a peak's width its prominence and bases must be
     known. You can supply these yourself with the argument `prominence_data`.
-    Otherwise they are internally calculated (see `peak_prominences`).
+    Otherwise, they are internally calculated (see `peak_prominences`).
 
     .. versionadded:: 1.1.0
 
@@ -685,7 +682,7 @@ def _select_by_peak_threshold(x, peaks, tmin, tmax):
     Parameters
     ----------
     x : ndarray
-        A one-dimensional array which is indexable by `peaks`.
+        A 1-D array which is indexable by `peaks`.
     peaks : ndarray
         Indices of peaks in `x`.
     tmin, tmax : scalar or ndarray or None
@@ -729,8 +726,8 @@ def find_peaks(x, height=None, threshold=None, distance=None,
     """
     Find peaks inside a signal based on peak properties.
 
-    This function takes a one-dimensional array and finds all local maxima by
-    simple comparison of neighbouring values. Optionally, a subset of these
+    This function takes a 1-D array and finds all local maxima by
+    simple comparison of neighboring values. Optionally, a subset of these
     peaks can be selected by specifying conditions for a peak's properties.
 
     Parameters
@@ -743,14 +740,15 @@ def find_peaks(x, height=None, threshold=None, distance=None,
         always interpreted as the  minimal and the second, if supplied, as the
         maximal required height.
     threshold : number or ndarray or sequence, optional
-        Required threshold of peaks, the vertical distance to its neighbouring
+        Required threshold of peaks, the vertical distance to its neighboring
         samples. Either a number, ``None``, an array matching `x` or a
         2-element sequence of the former. The first element is always
         interpreted as the  minimal and the second, if supplied, as the maximal
         required threshold.
     distance : number, optional
         Required minimal horizontal distance (>= 1) in samples between
-        neighbouring peaks. The removal order is explained in the notes section.
+        neighbouring peaks. Smaller peaks are removed first until the condition
+        is fulfilled for all remaining peaks.
     prominence : number or ndarray or sequence, optional
         Required prominence of peaks. Either a number, ``None``, an array
         matching `x` or a 2-element sequence of the former. The first
@@ -760,7 +758,7 @@ def find_peaks(x, height=None, threshold=None, distance=None,
         Required width of peaks in samples. Either a number, ``None``, an array
         matching `x` or a 2-element sequence of the former. The first
         element is always interpreted as the  minimal and the second, if
-        supplied, as the maximal required prominence.
+        supplied, as the maximal required width.
     wlen : int, optional
         Used for calculation of the peaks prominences, thus it is only used if
         one of the arguments `prominence` or `width` is given. See argument
@@ -842,7 +840,7 @@ def find_peaks(x, height=None, threshold=None, distance=None,
     Some additional comments on specifying conditions:
 
     * Almost all conditions (excluding `distance`) can be given as half-open or
-      closed intervals, e.g ``1`` or ``(1, None)`` defines the half-open
+      closed intervals, e.g., ``1`` or ``(1, None)`` defines the half-open
       interval :math:`[1, \\infty]` while ``(None, 1)`` defines the interval
       :math:`[-\\infty, 1]`. The open interval ``(None, None)`` can be specified
       as well, which returns the matching properties without exclusion of peaks.
@@ -854,9 +852,8 @@ def find_peaks(x, height=None, threshold=None, distance=None,
       `height`, `threshold`, `distance`, `prominence`, `width`. In most cases
       this order is the fastest one because faster operations are applied first
       to reduce the number of peaks that need to be evaluated later.
-    * Satisfying the distance condition is accomplished by iterating over all
-      peaks in descending order based on their height and removing all lower
-      peaks that are too close.
+    * While indices in `peaks` are guaranteed to be at least `distance` samples
+      apart, edges of flat peaks may be closer than the allowed `distance`.
     * Use `wlen` to reduce the time it takes to evaluate the conditions for
       `prominence` or `width` if `x` is large or has many local maxima
       (see `peak_prominences`).
@@ -892,7 +889,7 @@ def find_peaks(x, height=None, threshold=None, distance=None,
     >>> plt.show()
 
     Another useful condition for periodic signals can be given with the
-    `distance` argument. In this case we can easily select the positions of
+    `distance` argument. In this case, we can easily select the positions of
     QRS complexes within the electrocardiogram (ECG) by demanding a distance of
     at least 150 samples.
 
@@ -904,8 +901,8 @@ def find_peaks(x, height=None, threshold=None, distance=None,
     >>> plt.show()
 
     Especially for noisy signals peaks can be easily grouped by their
-    prominence (see `peak_prominences`). E.g. we can select all peaks except
-    for the mentioned QRS complexes by limiting the allowed prominenence to 0.6.
+    prominence (see `peak_prominences`). E.g., we can select all peaks except
+    for the mentioned QRS complexes by limiting the allowed prominence to 0.6.
 
     >>> peaks, properties = find_peaks(x, prominence=(None, 0.6))
     >>> properties["prominences"].max()
@@ -914,8 +911,8 @@ def find_peaks(x, height=None, threshold=None, distance=None,
     >>> plt.plot(peaks, x[peaks], "x")
     >>> plt.show()
 
-    And finally let's examine a different section of the ECG which contains
-    beat forms of different shape. To select only the atypical heart beats we
+    And, finally, let's examine a different section of the ECG which contains
+    beat forms of different shape. To select only the atypical heart beats, we
     combine two conditions: a minimal prominence of 1 and width of at least 20
     samples.
 
@@ -1037,9 +1034,8 @@ def _identify_ridge_lines(matr, max_distances, gap_thresh):
 
     References
     ----------
-    Bioinformatics (2006) 22 (17): 2059-2065.
-    :doi:`10.1093/bioinformatics/btl355`
-    http://bioinformatics.oxfordjournals.org/content/22/17/2059.long
+    .. [1] Bioinformatics (2006) 22 (17): 2059-2065.
+       :doi:`10.1093/bioinformatics/btl355`
 
     Examples
     --------
@@ -1109,7 +1105,7 @@ def _identify_ridge_lines(matr, max_distances, gap_thresh):
         # XXX Modifying a list while iterating over it.
         # Should be safe, since we iterate backwards, but
         # still tacky.
-        for ind in xrange(len(ridge_lines) - 1, -1, -1):
+        for ind in range(len(ridge_lines) - 1, -1, -1):
             line = ridge_lines[ind]
             if line[2] > gap_thresh:
                 final_lines.append(line)
@@ -1157,8 +1153,8 @@ def _filter_ridge_lines(cwt, ridge_lines, window_size=None, min_length=None,
 
     References
     ----------
-    Bioinformatics (2006) 22 (17): 2059-2065. :doi:`10.1093/bioinformatics/btl355`
-    http://bioinformatics.oxfordjournals.org/content/22/17/2059.long
+    .. [1] Bioinformatics (2006) 22 (17): 2059-2065.
+       :doi:`10.1093/bioinformatics/btl355`
 
     """
     num_points = cwt.shape[1]
@@ -1172,7 +1168,7 @@ def _filter_ridge_lines(cwt, ridge_lines, window_size=None, min_length=None,
 
     # Filter based on SNR
     row_one = cwt[0, :]
-    noises = np.zeros_like(row_one)
+    noises = np.empty_like(row_one)
     for ind, val in enumerate(row_one):
         window_start = max(ind - hf_window, 0)
         window_end = min(ind + hf_window + odd, num_points)
@@ -1191,7 +1187,8 @@ def _filter_ridge_lines(cwt, ridge_lines, window_size=None, min_length=None,
 
 
 def find_peaks_cwt(vector, widths, wavelet=None, max_distances=None,
-                   gap_thresh=None, min_length=None, min_snr=1, noise_perc=10):
+                   gap_thresh=None, min_length=None,
+                   min_snr=1, noise_perc=10, window_size=None):
     """
     Find peaks in a 1-D array with wavelet transformation.
 
@@ -1234,6 +1231,9 @@ def find_peaks_cwt(vector, widths, wavelet=None, max_distances=None,
         When calculating the noise floor, percentile of data points
         examined below which to consider noise. Calculated using
         `stats.scoreatpercentile`.  Default is 10.
+    window_size : int, optional
+        Size of window to use to calculate noise floor.
+        Default is ``cwt.shape[1] / 20``.
 
     Returns
     -------
@@ -1257,7 +1257,7 @@ def find_peaks_cwt(vector, widths, wavelet=None, max_distances=None,
     The algorithm is as follows:
      1. Perform a continuous wavelet transform on `vector`, for the supplied
         `widths`. This is a convolution of `vector` with `wavelet(width)` for
-        each width in `widths`. See `cwt`
+        each width in `widths`. See `cwt`.
      2. Identify "ridge lines" in the cwt matrix. These are relative maxima
         at each row, connected across adjacent rows. See identify_ridge_lines
      3. Filter the ridge_lines using filter_ridge_lines.
@@ -1267,8 +1267,7 @@ def find_peaks_cwt(vector, widths, wavelet=None, max_distances=None,
     References
     ----------
     .. [1] Bioinformatics (2006) 22 (17): 2059-2065.
-        :doi:`10.1093/bioinformatics/btl355`
-        http://bioinformatics.oxfordjournals.org/content/22/17/2059.long
+       :doi:`10.1093/bioinformatics/btl355`
 
     Examples
     --------
@@ -1289,10 +1288,11 @@ def find_peaks_cwt(vector, widths, wavelet=None, max_distances=None,
     if wavelet is None:
         wavelet = ricker
 
-    cwt_dat = cwt(vector, wavelet, widths)
+    cwt_dat = cwt(vector, wavelet, widths, window_size=window_size)
     ridge_lines = _identify_ridge_lines(cwt_dat, max_distances, gap_thresh)
     filtered = _filter_ridge_lines(cwt_dat, ridge_lines, min_length=min_length,
-                                   min_snr=min_snr, noise_perc=noise_perc)
+                                   window_size=window_size, min_snr=min_snr,
+                                   noise_perc=noise_perc)
     max_locs = np.asarray([x[1][0] for x in filtered])
     max_locs.sort()
 
